package SearchResultsCFListing::Entry;

use strict;

use MT::Util qw( format_ts relative_date remove_html encode_js break_up_text first_n_words );
use MT::I18N qw( const );
use CustomFields::Util qw( get_meta );

use Data::Dumper;

sub add_field {
    my ($cb, $app, $template) = @_;
    my $blog = $app->blog;
    my $type = $app->param('__mode');
    $type =~ s/list_(.*)/$1/;
    $type = 'entry';

    my $plugin = MT->component("SearchResultsCFListing");
    return unless $plugin->get_config_value('use_plugin_entry', 'blog:'.$blog->id);

    # display only blog and this_website.
#    return if !($blog->parent_id) && $app->param('filter_key') ne 'this_website';

    my $th = '';
    my $td = '';

    my $excerpt_head = MT->translate('Excerpt');
    my $keywords_head = MT->translate('Keywords');
    my $tags_head = MT->translate('Tags');

#    my $display_excerpt = $plugin->get_config_value('display_excerpt', 'blog:' . $blog->id);
    my $display_keywords = $plugin->get_config_value('display_keywords', 'blog:' . $blog->id);
    my $display_tags = $plugin->get_config_value('display_tags', 'blog:' . $blog->id);
    my $display_entry_cf = $plugin->get_config_value('display_entry_cf', 'blog:' . $blog->id);

    my $counter = 0;

    # excerpt
#    if ($display_excerpt) {
#        $th .= qq{<th class="col head excerpt"><span class="col-label first-child last-child">$excerpt_head</span></th>};
#        $td .= '<td class="col"><mt:var name="excerpt" /></td>';
#        $counter++;
#    }

    # keywords
    if ($display_keywords) {
        $th .= qq{<th class="col head keywords"><span class="col-label first-child last-child">$keywords_head</span></th>};
        $td .= '<td class="col"><mt:var name="keywords" /></td>';
        $counter++;
    }

    # tags
    if ($display_tags) {
        $th .= qq{<th class="col head tags"><span class="col-label first-child last-child">$tags_head</span></th>};
        $td .= '<td class="col"><mt:var name="tags" /></td>';
        $counter++;
    }

    if ($display_entry_cf) {
        my $terms = { blog_id => $blog->id, obj_type => $type };
        my $iter = CustomFields::Field->load_iter($terms);
        while ( my $field = $iter->() ) {
            next if $field->type =~ /embed|image|audio|video|file/;
            $th .= '<th class="col head custom-prefs-'.$field->basename.'"><span class="col-label first-child last-child">'. $field->name . '</span></th>';
#            if ($field->type eq 'image') {
#                $td .= '<td class="col custom-prefs-'.$field->basename.'"><mt:var name="'. $field->basename . '" mteval="1" /></td>';
#            } else {
                $td .= '<td class="col custom-prefs-'.$field->basename.'"><mt:var name="'. $field->basename . '" escape="html" mteval="1" /></td>';
#            }
            $counter++;
        }
    }

    my $old = <<HTML;
<mtapp:listing  hide_pager="1">
HTML
    $old = quotemeta($old);

    my $new = <<HTML;
<style>
.col.title {
    width: 80px!important;
}
</style>
<mtapp:listing  hide_pager="1">
HTML
    $$template =~ s/$old/$new/;

    $old = <<HTML;
      <td class="col title" colspan="<mt:if name="is_blog">5<mt:else>6</mt:if>">
HTML
    $old = quotemeta($old);

    my $page_blog = $counter + 5;
    my $page_website = $counter + 6;
    my $entry_blog = $counter + 5;
    my $entry_website = $counter + 6;

    $new = <<HTML;
      <td class="col title" colspan="<mt:if name="is_blog">${entry_blog}<mt:else>${entry_website}</mt:if>">
>
HTML
    $$template =~ s/$old/$new/;

    $old = <<HTML;
    </tr>
    </mt:setvarblock>
HTML
    $old = quotemeta($old);

    $new = <<HTML;
$th
    </tr>
    </mt:setvarblock>
HTML
    $$template =~ s/$old/$new/;

    $old = <<HTML;
    </tr>
  <mt:if name="__last__">
  </tbody>
HTML
    $old = quotemeta($old);

    $new = <<HTML;
$td
    </tr>
  <mt:if name="__last__">
  </tbody>
HTML
    $$template =~ s/$old/$new/;
}

sub build_entry_table {
    my $app = shift;
    my (%args) = @_;

    my $app_author = $app->user;
    my $perms      = $app->permissions;
    my $type       = $args{type};
    my $class      = $app->model($type);

    my $list_pref = $app->list_pref($type);
    if ( $args{is_power_edit} ) {
        delete $list_pref->{view_expanded};
    }
    my $iter;
    if ( $args{load_args} ) {
        $iter = $class->load_iter( @{ $args{load_args} } );
    }
    elsif ( $args{iter} ) {
        $iter = $args{iter};
    }
    elsif ( $args{items} ) {
        $iter = sub { shift @{ $args{items} } };
    }
    return [] unless $iter;
    my $limit         = $args{limit};
    my $is_power_edit = $args{is_power_edit} || 0;
    my $param         = $args{param} || {};

    ## Load list of categories for display in filter pulldown (and selection
    ## pulldown on power edit page).
    my ( $c_data, %cats );
    my $blog_id = $app->param('blog_id');
    if ($blog_id) {
        $c_data = $app->_build_category_list(
            blog_id => $blog_id,
            type    => $class->container_type,
        );
        my $i = 0;
        for my $row (@$c_data) {
            $row->{category_index} = $i++;
            my $spacer = $row->{category_label_spacer} || '';
            $spacer =~ s/\&nbsp;/\\u00A0/g;
            $row->{category_label_js}
                = $spacer . encode_js( $row->{category_label} );
            $cats{ $row->{category_id} } = $row;
        }
        $param->{category_loop} = $c_data;
    }

    my ( $date_format, $datetime_format );

    if ($is_power_edit) {
        $date_format     = "%Y.%m.%d";
        $datetime_format = "%Y-%m-%d %H:%M:%S";
    }
    else {
        $date_format     = MT::App::CMS::LISTING_DATE_FORMAT();
        $datetime_format = MT::App::CMS::LISTING_DATETIME_FORMAT();
    }

    my @cat_list;
    if ($is_power_edit) {
        @cat_list = sort {
            $cats{$a}->{category_index} <=> $cats{$b}->{category_index}
            }
            keys %cats;
    }

    my @data;
    my %blogs;
    require MT::Blog;
    my $title_max_len = const('DISPLAY_LENGTH_EDIT_ENTRY_TITLE');
    my $excerpt_max_len
        = const('DISPLAY_LENGTH_EDIT_ENTRY_TEXT_FROM_EXCERPT');
    my $text_max_len = const('DISPLAY_LENGTH_EDIT_ENTRY_TEXT_BREAK_UP');
    my %blog_perms;
    $blog_perms{ $perms->blog_id } = $perms if $perms;

    while ( my $obj = $iter->() ) {
        my $blog_perms;
        if ( !$app_author->is_superuser() ) {
            $blog_perms = $blog_perms{ $obj->blog_id }
                || $app_author->blog_perm( $obj->blog_id );
        }

        my $row = $obj->get_values;
        $row->{text} ||= '';
        if ( my $ts = $obj->authored_on ) {
            $row->{created_on_formatted}
                = format_ts( $date_format, $ts, $obj->blog,
                $app->user ? $app->user->preferred_language : undef );
            $row->{created_on_time_formatted}
                = format_ts( $datetime_format, $ts, $obj->blog,
                $app->user ? $app->user->preferred_language : undef );
            $row->{created_on_relative}
                = relative_date( $ts, time, $obj->blog );
        }
        if ( my $ts = $obj->modified_on ) {
            $row->{modified_on_formatted}
                = format_ts( $date_format, $ts, $obj->blog,
                $app->user ? $app->user->preferred_language : undef );
            $row->{modified_on_time_formatted}
                = format_ts( $datetime_format, $ts, $obj->blog,
                $app->user ? $app->user->preferred_language : undef );
            $row->{modified_on_relative}
                = relative_date( $ts, time, $obj->blog );
        }
        my $author = $obj->author;
        $row->{author_name}
            = $author ? $author->name : $app->translate('(user deleted)');
        if ( my $cat = $obj->category ) {
            $row->{category_label}    = $cat->label;
            $row->{category_basename} = $cat->basename;
            $row->{category_id}       = $cat->id;
        }
        else {
            $row->{category_label}    = '';
            $row->{category_basename} = '';
            $row->{category_id}       = '';
        }
        $row->{file_extension} = $obj->blog ? $obj->blog->file_extension : '';
        $row->{title_short} = $obj->title;
        if ( !defined( $row->{title_short} ) || $row->{title_short} eq '' ) {
            my $title = remove_html( $obj->text );
            $row->{title_short}
                = substr( defined($title) ? $title : "", 0, $title_max_len )
                . '...';
        }
        else {
            $row->{title_short} = remove_html( $row->{title_short} );
            $row->{title_short}
                = substr( $row->{title_short}, 0, $title_max_len + 3 ) . '...'
                if length( $row->{title_short} ) > $title_max_len;
        }

        $row->{tags} = MT::Tag->join( ',', $obj->tags );

        if ( $row->{excerpt} ) {
            $row->{excerpt} = remove_html( $row->{excerpt} );
        }
        if ( !$row->{excerpt} ) {
            my $text = remove_html( $row->{text} ) || '';
            $row->{excerpt} = first_n_words( $text, $excerpt_max_len );
            if ( length($text) > length( $row->{excerpt} ) ) {
                $row->{excerpt} .= ' ...';
            }
        }
        $row->{text} = break_up_text( $row->{text}, $text_max_len )
            if $row->{text};
        $row->{title_long} = remove_html( $obj->title );
        $row->{status_text}
            = $app->translate( MT::Entry::status_text( $obj->status ) );
        $row->{ "status_" . MT::Entry::status_text( $obj->status ) } = 1;
        $row->{has_edit_access} = $app_author->is_superuser
            || ( ( 'entry' eq $type )
            && $blog_perms
            && $blog_perms->can_edit_entry( $obj, $app_author ) )
            || (
               ( 'page' eq $type )
            && $blog_perms
            && (  $app_author->id == $obj->author_id
                ? $blog_perms->can_do('edit_own_page')
                : $blog_perms->can_do('edit_all_pages')
            )
            );
        if ($is_power_edit) {
            $row->{has_publish_access} = $app_author->is_superuser
                || ( $blog_perms
                && $blog_perms->can_edit_entry( $obj, $app_author, 1 ) );
            $row->{is_editable} = $row->{has_edit_access};

            ## This is annoying. In order to generate and pre-select the
            ## category, user, and status pull down menus, we need to
            ## have a separate *copy* of the list of categories and
            ## users for every entry listed, so that each row in the list
            ## can "know" whether it is selected for this entry or not.
            my @this_c_data;
            my $this_category_id
                = $obj->category ? $obj->category->id : undef;
            for my $c_id (@cat_list) {
                push @this_c_data, { %{ $cats{$c_id} } };
                $this_c_data[-1]{category_is_selected} = $this_category_id
                    && $this_category_id == $c_id ? 1 : 0;
            }
            $row->{row_category_loop} = \@this_c_data;

            if ( $obj->author ) {
                $row->{row_author_name} = $obj->author->name;
                $row->{row_author_id}   = $obj->author->id;
            }
            else {
                $row->{row_author_name}
                    = $app->translate( '(user deleted - ID:[_1])',
                    $obj->author_id );
                $row->{row_author_id} = $obj->author_id,;
            }

            $row->{entry_blog_id} = $obj->blog_id;
        }
        if ( my $blog = $blogs{ $obj->blog_id }
            ||= MT::Blog->load( $obj->blog_id ) )
        {
            $row->{weblog_id}   = $blog->id;
            $row->{weblog_name} = $blog->name;
        }
        if ( $obj->status == MT::Entry::RELEASE() ) {
            $row->{entry_permalink} = MT::Util::encode_html( $obj->permalink );
        }
        $row->{object} = $obj;
        push @data, $row;
    }
    return [] unless @data;

    $param->{entry_table}[0] = {%$list_pref};
    $param->{object_loop} = $param->{entry_table}[0]{object_loop} = \@data;

    my $data = $param->{object_loop};

    # display options
    my $type = $app->param('__mode');
    $type =~ s/list_(.*)/$1/;
    $type = 'entry';

    my $plugin = MT->component("SearchResultsCFListing");
    my $display_entry_cf = $plugin->get_config_value('display_entry_cf', 'blog:' . $blog_id);
    if ($display_entry_cf) {
        foreach my $row (@$data) {
            my $entry = MT::Entry->load($row->{id});
            my $meta = get_meta($entry);
            foreach my $basename (keys %$meta) {
                $row->{$basename} = $meta->{$basename};
                if ($meta->{$basename} =~ /^<form mt:asset-id="(\d+)" class="mt-enclosure mt-enclosure-image" style="display: inline;"><a href="(.*)">.*$/) {
                    $row->{$basename} = '<span class="gallery"><mt:Asset id="'.$1.'"><mt:AssetThumbnailLink width="75" square="1"></mt:Asset></span>';
                }
            }
        }
    }

#    my $list = $type . $app->param('blog_id');
#    my $cookie = $app->cookie_val('cf_list_pref') || '';
#    my @list_prefs = split /;/, $cookie;
#    foreach my $pref (@list_prefs) {
#        my ( $name, $prefs ) = $pref =~ m/^(\w+):(.*)$/;
#        next unless $name && $prefs;
#        if ( $name eq $list ) {
#            my @prefs = split /,/, $prefs;
#            foreach (@prefs) {
#                my ( $k, $v ) = split /=/;
#                $param->{'custom_prefs-'.$k} = $v;
#            }
#        }
#    }

    $app->load_list_actions( $type, \%$param )
        unless $is_power_edit;
    \@data;
}

1;
